-- Colored names!
-- Written by Wolfy, uses PlayerMsg Framework as a base

-- Colors:

rawset(_G, "colors", {}) -- Define a global namespace for the colors.

colors = { -- I yanked this right out of Terminal.
	-- proper names
	white  = "\x80",
	purple = "\x81",
	yellow = "\x82",
	green  = "\x83",
	blue   = "\x84",
	red    = "\x85",
	grey   = "\x86",
	orange = "\x87",
	
	-- indexes
	[1] = "\x80",
	[2] = "\x81",
	[3] = "\x82",
	[4] = "\x83",
	[5] = "\x84",
	[6] = "\x85",
	[7] = "\x86",
	[8] = "\x87",
}

local function SkincolorToColor(color) -- Used for auto. Generates name colors based on skin color.
	if (color == SKINCOLOR_WHITE or color == SKINCOLOR_NONE) then
		return colors.white
	elseif (color >= SKINCOLOR_SILVER and color <= SKINCOLOR_BLACK) then
		return colors.grey
	elseif (color >= SKINCOLOR_CYAN and color <= SKINCOLOR_BLUE) then
		return colors.blue
	elseif ((color >= SKINCOLOR_PEACH and color <= SKINCOLOR_PINK) or (color == SKINCOLOR_RED or color == SKINCOLOR_DARKRED)) then
		return colors.red
	elseif (color == SKINCOLOR_LAVENDER or color == SKINCOLOR_PURPLE) then
		return colors.purple
	elseif (color >= SKINCOLOR_ORANGE and color <= SKINCOLOR_BROWN) then
		return colors.orange
	elseif (color >= SKINCOLOR_NEONGREEN and color <= SKINCOLOR_ZIM) then
		return colors.green
	elseif (color >= SKINCOLOR_OLIVE and color <= SKINCOLOR_GOLD) then
		return colors.yellow
	else
		return colors.white
	end
end

local localplayercolor = 1 -- Set a color for the local player.
local stringcolor = "white" -- I don't know what the hell I'm doing anymore.

for player in players.iterate do -- Set the default value for namecolor
	player.namecolor = 1
end

COM_AddCommand("namecolor", function(player, color) -- Kind of a hack. Used as a fake cvar, the actual value is in the player struct.
	if not color then CONS_Printf(player, [["namecolor" is "]]..stringcolor..[[" default is "white"]]) return end
	if color == "white" then player.namecolor = 1
	elseif color == "purple" then player.namecolor = 2
	elseif color == "yellow" then player.namecolor = 3
	elseif color == "green" then player.namecolor = 4
	elseif color == "blue" then player.namecolor = 5
	elseif color == "red" then player.namecolor = 6
	elseif color == "grey" then player.namecolor = 7
	elseif color == "orange" then player.namecolor = 8
	else CONS_Printf(player, [["]]..color..[[" is not a possible value for "namecolor"]]) return
	end
	stringcolor = color
	localplayercolor = player.namecolor
end)

local cv_nameoptions = CV_RegisterVar({"nameoptions", "auto", 0, {auto = 0, force = 1, perplayer = 2}})
local cv_forceteamcolor = CV_RegisterVar({"forceteamcolor", "Yes", 0, CV_YesNo})
--local cv_namecolor = CV_RegisterVar({"namecolor", "white", 0, {white = 1, purple = 2, yellow = 3, green = 4, blue = 5, red = 6, grey = 7, orange = 8}})

-- Player symbol management
local function getSymbol(player)
	if player == server then return "~" end -- Server
	if ((player == admin) and not (player == server)) then return "@" end -- Admin
	return ""
end

-- Function for retrieving the current team color
local function getTeamColor(player) 
	if G_GametypeHasTeams() then
		if player.ctfteam == 0 then return colors.white end
		if player.ctfteam == 1 then return colors.red end 
		if player.ctfteam == 2 then return colors.blue end 
	else return ""
	end
end

-- Grabs names in their pre-display form, meaning without the <>s and stuff but with their color and symbol.
local function getName(player) 
	if cv_forceteamcolor.value == 1 and G_GametypeHasTeams() then
		return getTeamColor(player)..getSymbol(player)..player.name..colors.white
	end
	if cv_nameoptions.value == 0 then -- auto
		return SkincolorToColor(player.skincolor)..getSymbol(player)..player.name..colors.white
	elseif cv_nameoptions.value == 1 then -- force
		return colors[localplayercolor]..getSymbol(player)..player.name..colors.white
	elseif cv_nameoptions.value == 2 and player.namecolor then -- perplayer
		return colors[player.namecolor]..getSymbol(player)..player.name..colors.white
	end
	return getTeamColor(player)..getSymbol(player)..player.name..colors.white
end

-- Builds the name according to the message type
local function builtName(player, msgtype, ...)
	if ... then -- Check if the player is using /me
		return colors.yellow.."* "..getName(player)..colors.yellow..""
	end
	if msgtype == 0 then
		return "<"..getName(player).."> "
	elseif msgtype == 1 then
		return ">>"..getName(player).."<< (team) "
	elseif msgtype == 2 then
		return "->*"..getName(player).."* "
	elseif msgtype == 4 then
		return "*"..getName(player).."* "
	else return
	end
end

-- Manage the player names
addHook("PlayerMsg", function(source, msgtype, target, message)
	if msgtype == 3 then return false end
	if msgtype == 0 then -- say
		if (message:sub(1,3) == "/me") then
			print(builtName(source, msgtype, true)..message:sub(4))
		else
			print(builtName(source, msgtype)..message)
		end
		S_StartSound(nil, sfx_radio)
	elseif msgtype == 1 then -- sayteam
		for player in players.iterate do
			if player.ctfteam == source.ctfteam then
				CONS_Printf(player, builtName(source, msgtype)..message)
				S_StartSound(nil, sfx_radio, player)
			end
		end
	elseif msgtype == 2 then -- sayto
		CONS_Printf(source, builtName(target, msgtype)..message)
		CONS_Printf(target, builtName(source, 4)..message)
		S_StartSound(nil, sfx_radio, source)
		S_StartSound(nil, sfx_radio, target)
	end
	return true -- Returning true will override the default messages with the changes
end)